/******************************************************************
 *****																												*****
 *****	Name: cs8900.c																				*****
 *****	Ver.: 1.0																						*****
 *****	Date: 07/05/2001																			*****
 *****	Auth: Andreas Dannenberg															*****
 *****				HTWK Leipzig																		*****
 *****				university of applied sciences									*****
 *****				Germany																				*****
 *****	Func: ethernet packet-driver for use with LAN-				*****
 *****				controller CS8900 from Crystal/Cirrus Logic		*****
 *****																												*****
 *****	Keil: Module modified for use with Philips						*****
 *****				LPC17xx EMAC Ethernet controller								*****
 *****																												*****
 ******************************************************************/

//    Modifications by Coridium Corp

#include "pcb.h"


// external define to remove compiler warnings
int printf(const char *format, ...);


#define MDIO	0x00000200
#define MDC		0x00000100

//--------------------------- write_PHY -------------------------------------

static void write_PHY (unsigned int PhyReg, unsigned short Value) 
{
	// Write a data 'Value' to PHY register 'PhyReg'. 
	unsigned long timer;

	// Hardware MII Management for LPC176x devices. 
	LPC_EMAC->MADR = DP83848C_DEF_ADR | PhyReg;
	LPC_EMAC->MWTD = Value;

	// Wait untill operation completed, typically 1us or less 
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 2000) //give it up to 2ms
		if ((LPC_EMAC->MIND & MIND_BUSY) == 0)
			break;
	
	if((LPC_TIM0->TC - timer) >= 2000)//should never happen
		printf("\r\nERROR: Timeout Writing Phy Reg\r\n");

	//printf("write_PHY %d us\r\n", (int)(LPC_TIM0->TC - timer) );//if you print before checking error you cause a false error to report
}


//--------------------------- read_PHY --------------------------------------

static unsigned short read_PHY (unsigned int PhyReg) 
{
	// Read a PHY register 'PhyReg'. 
	unsigned int val;
	unsigned long timer;

	LPC_EMAC->MADR = DP83848C_DEF_ADR | PhyReg;
	LPC_EMAC->MCMD = MCMD_READ;

	// Wait until operation completed, typically 1us or less 
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 2000) //give it up to 2ms
		if ((LPC_EMAC->MIND & MIND_BUSY) == 0)
			break;
	
	if( (LPC_TIM0->TC - timer) >= 2000)//should never happen
		printf("\r\nERROR: Timeout Reading Phy Reg\r\n");

	//printf("read_PHY %d us\r\n", (int)(LPC_TIM0->TC - timer));//if you print before checking error you cause a false error to report

	LPC_EMAC->MCMD = 0;
	val = LPC_EMAC->MRDD;

	return (val);
}

// Keil: function added to initialize Rx Descriptors
void rx_descr_init (void)
{
	unsigned int i;

	for (i = 0; i < NUM_RX_FRAG; i++) {
		RX_DESC_PACKET(i)	= RX_BUF(i);
		RX_DESC_CTRL(i)		= RCTRL_INT | (ETH_FRAG_SIZE-1);
		RX_STAT_INFO(i)		= 0;
		RX_STAT_HASHCRC(i) = 0;
	}

	// Set EMAC Receive Descriptor Registers. 
	LPC_EMAC->RxDescriptor		= RX_DESC_BASE;
	LPC_EMAC->RxStatus				= RX_STAT_BASE;
	LPC_EMAC->RxDescriptorNumber = NUM_RX_FRAG-1;

	// Rx Descriptors Point to 0 
	LPC_EMAC->RxConsumeIndex	= 0;
}


// Keil: function added to initialize Tx Descriptors
void tx_descr_init (void) 
{
	unsigned int i;

	for (i = 0; i < NUM_TX_FRAG; i++) {
		TX_DESC_PACKET(i) = TX_BUF(i);
		TX_DESC_CTRL(i)	= 0;
		TX_STAT_INFO(i)	= 0;
	}

	// Set EMAC Transmit Descriptor Registers. 
	LPC_EMAC->TxDescriptor		= TX_DESC_BASE;
	LPC_EMAC->TxStatus				= TX_STAT_BASE;
	LPC_EMAC->TxDescriptorNumber = NUM_TX_FRAG-1;

	// Tx Descriptors Point to 0 
	LPC_EMAC->TxProduceIndex	= 0;
}


// configure port-pins for use with LAN-controller,
// reset it and send the configuration-sequence

// #define ETH_DBG
#ifdef ETH_DBG
unsigned int TxRequested, TxComplete;
#endif

//check ethernet status, it should have been already initialized
unsigned char Found_Ethernet(void)
{
	unsigned int id1,id2;

	// Check if this is a DP83848C PHY. 
	id1 = read_PHY (PHY_REG_IDR1);
	id2 = read_PHY (PHY_REG_IDR2);
	if (((id1 << 16) | (id2 & 0xFFF0)) == DP83848C_ID)
	{
		if ( (read_PHY (PHY_REG_BMSR) & 0x0020) && (read_PHY (PHY_REG_STS) & 0x0001) )// Autonegotiation Complete and Link is on
		{
			//printf("Its all good\r\n");
			return 1;
		}
		
		else
			printf("Ethernet not connected\r\n");
	} else {
		printf("Ethernet controller not found\r\n");
	}
	return 0;
}

extern unsigned char MyMAC[6];

// Initializes the EMAC ethernet controller
void Init_EMAC(void)
{
	unsigned int regv,id1,id2;
	unsigned long timer;
#ifdef ETH_DBG
	TxRequested = TxComplete = 0;
#endif
	
	if (GetMacAddr() == -1) {
		printf("GetMacAddr Failed\r\n");
		return;
	}
	
	regv = 0; //initialize for compiler warning
	
	// Power Up the EMAC controller. 
	LPC_SC->PCONP |= 0x40000000;
	
	LPC_PINCON->PINSEL2 = 0x50150105;
	LPC_PINCON->PINSEL3 &= ~0x0000000F;
	LPC_PINCON->PINSEL3 |= 0x00000005;

	// Reset all EMAC internal modules. 
	LPC_EMAC->MAC1 = MAC1_RES_TX | MAC1_RES_MCS_TX | MAC1_RES_RX | MAC1_RES_MCS_RX |
						MAC1_SIM_RES | MAC1_SOFT_RES;
	LPC_EMAC->Command = CR_REG_RES | CR_TX_RES | CR_RX_RES;

	// A short delay after reset. 
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 5); //wait 5us

	// Initialize MAC control registers. 
	LPC_EMAC->MAC1 = MAC1_PASS_ALL;
	LPC_EMAC->MAC2 = MAC2_CRC_EN | MAC2_PAD_EN;
	LPC_EMAC->MAXF = ETH_MAX_FLEN;
	LPC_EMAC->CLRT = CLRT_DEF;
	LPC_EMAC->IPGR = IPGR_DEF;

	// Enable Reduced MII interface. 
	LPC_EMAC->Command = CR_RMII | CR_PASS_RUNT_FRM;

	// Reset Reduced MII Logic. 
	LPC_EMAC->SUPP = SUPP_RES_RMII;
	// A short delay 
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 5); //wait 5us
	LPC_EMAC->SUPP = 0;

	// Put the DP83848C in reset mode 
	write_PHY (PHY_REG_BMCR, 0x8000);

	// Wait for hardware reset to end. 
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 2000) //give it up to 2ms, usually takes about 0us
	{
		regv = read_PHY (PHY_REG_BMCR);
		if (!(regv & 0x8000))
			// Reset complete 
			break;
	}
	//printf("PHY_REG_BMCR 0x%04X %dus\r\n",regv, (int)(LPC_TIM0->TC - timer));

	// Check if this is a DP83848C PHY. 
	id1 = read_PHY (PHY_REG_IDR1);
	id2 = read_PHY (PHY_REG_IDR2);
	if (((id1 << 16) | (id2 & 0xFFF0)) == DP83848C_ID)
	{
		// Configure the PHY device 
		// Use autonegotiation about the link speed. 
		write_PHY (PHY_REG_BMCR, PHY_AUTO_NEG);
		
		// Wait to complete Auto_Negotiation. 
		timer = LPC_TIM0->TC;
		while( (LPC_TIM0->TC - timer) < 5000000) //give it up to 5 sec, usually takes around 2 sec
		{
			regv = read_PHY (PHY_REG_BMSR);
			if (regv & 0x0020)
				// Autonegotiation Complete. 
				break;
		}
	}
	//printf("PHY_REG_BMSR 0x%04X %dus\r\n",regv, (int)(LPC_TIM0->TC - timer));

	// Check the link status.
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 2000) //give it up to 2ms, usually takes about 0us
	{
		regv = read_PHY (PHY_REG_STS);
		if (regv & 0x0001)
			// Link is on. 
			break;
	}
	//printf("PHY_REG_STS 0x%04X %dus\r\n",regv, (int)(LPC_TIM0->TC - timer));
	
	// Configure Full/Half Duplex mode. 
	if (regv & 0x0004) {
		// Full duplex is enabled. 
		LPC_EMAC->MAC2		|= MAC2_FULL_DUP;
		LPC_EMAC->Command |= CR_FULL_DUP;
		LPC_EMAC->IPGT		= IPGT_FULL_DUP;
	}
	else {
		// Half duplex mode. 
		LPC_EMAC->IPGT = IPGT_HALF_DUP;
	}

	// Configure 100MBit/10MBit mode. 
	if (regv & 0x0002) {
		// 10MBit mode. 
		LPC_EMAC->SUPP = 0;
	}
	else {
		// 100MBit mode. 
		LPC_EMAC->SUPP = SUPP_SPEED;
	}

	// Set the Ethernet MAC Address registers 
	LPC_EMAC->SA0 = (MyMAC[5] << 8) | MyMAC[4];
	LPC_EMAC->SA1 = (MyMAC[3] << 8) | MyMAC[2];
	LPC_EMAC->SA2 = (MyMAC[1] << 8) | MyMAC[0];

	// Initialize Tx and Rx DMA Descriptors 
	rx_descr_init ();
	tx_descr_init ();

	// Receive Broadcast and Perfect Match Packets 
	LPC_EMAC->RxFilterCtrl = RFC_BCAST_EN | RFC_PERFECT_EN;

#ifdef ETH_DBG
	// Enable EMAC interrupts, TX Done only. 
	LPC_EMAC->IntEnable = INT_TX_DONE;
	NVIC->ISER[0] = 1 << ENET_IRQn;// enable interrupt ENET
#endif	

	// Reset all interrupts 
	LPC_EMAC->IntClear	= 0xFFFF;

	// Enable receive and transmit mode of MAC Ethernet core 
	LPC_EMAC->Command	|= (CR_RX_EN | CR_TX_EN);
	LPC_EMAC->MAC1		|= MAC1_REC_EN;

	// first TX fails without this delay, the way to tell is if you get a different IP with each reset
	// 100 ms works for me but just to be sure we'll use 200 ms
	timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < 200000);//wait 200 ms
}

// If there is a frame available copy it to user RAM Dest and return 1, else return 0
unsigned char CheckFrameReceived(int Dest)						// Packet received ?
{
	int RxLen;
	unsigned int idx;
	int i;
	unsigned int *pDst, *pSrc; 

	//unsigned cpsr = disableIRQ();//feedTheDog
	//LPC_WDT->WDFEED = 0xaa;// reset watchdog timer
	//LPC_WDT->WDFEED = 0x55;
	//restoreIRQ(cpsr);

	if (LPC_EMAC->RxProduceIndex == LPC_EMAC->RxConsumeIndex)		// more packets received ?
		return(0);

	idx = LPC_EMAC->RxConsumeIndex;
	RxLen = (RX_STAT_INFO(idx) & RINFO_SIZE) - 3;//NOTE: Size is 0 based, -3 means don't count last 4 bytes of FCS checksum
	
	//clear Rx buffer - not really necessary, just for debug, takes 31us
	pDst = (unsigned int *)Dest;
	for (i = 0; i < (1518 >> 2); i++) *pDst++ = 0; // MAXRXFRAMEC = 1518

	pDst = (unsigned int *)Dest;
	pSrc = (unsigned int *)RX_DESC_PACKET(idx);

	for (i = 0; i < (RxLen+3)>>2; i++) //full frame takes 38us
		*pDst++ = *pSrc++;

	// DMA free packet. 
	idx = LPC_EMAC->RxConsumeIndex;
	if (++idx == NUM_RX_FRAG) idx = 0;
	LPC_EMAC->RxConsumeIndex = idx;

	return(RxLen);

}

// copies bytes from MCU-memory to frame port and starts hardware transmission
void CopyToFrame_EMAC(void *Source, unsigned int Size)
{
	unsigned int *pDst, *pSrc; 
	unsigned int pdx;
	int i;
	
	pdx	= LPC_EMAC->TxProduceIndex;
 
	//If the TxProduceIndex equals TxConsumeIndex - 1 the descriptor array is full and software
	//should stop producing new descriptors until hardware has transmitted some frames and
	//updated the TxConsumeIndex.

	// // NOTE : adding this check creates TCP retransmit problems, even after increasing NUM_TX_FRAG.
	// // But increasing NUM_TX_FRAG from 3 to 5 seems to solve a congestion problem I was seeing
	//unsigned int cdx,cdxM1;
	//cdx = LPC_EMAC->TxConsumeIndex;
	//if(cdx==0)cdxM1 = NUM_TX_FRAG-1;else cdxM1 = cdx-1;
	//if(pdx == cdxM1)
	//{
	//	printf("%d %d ",pdx,cdx );
	//timer = LPC_TIM0->TC;
	//while( (LPC_TIM0->TC - timer) < 30000) //give it up to 30ms
	//		if(LPC_EMAC->TxConsumeIndex != cdx)//wait for TxConsumeIndex to change 
	//			break;
	//				
	//	printf(" %d %d\r\n",(int)(LPC_TIM0->TC - timer),TxCount);
	//}

	
	//In this design I'm using the WHOLE 32K Ethernet RAM with NUM_RX_FRAG and NUM_TX_FRAG at 10
	//In the ARMweb version I wait here for TxReadyToSend but that's not necessary here even when using just 5 FRAGs
	//Besides, as I NOTE above, I can't get that check for TxProduceIndex and TxConsumeIndex to work right
	//And its not worth using interrupts below
	//I use interrupts as little as possible for code I'm providing to others
#ifdef ETH_DBG
	unsigned long timer;
	//this interrupt driven check for TxOverun works better, I never see overruns with NUM_TX_FRAG = 5 but a few at 3
	if(TxRequested > (TxComplete+NUM_TX_FRAG))
	{
		//printf("%d > %d ",TxRequested, TxComplete);
		timer = LPC_TIM0->TC;
		while( (LPC_TIM0->TC - timer) < 1000000) //give it up to 30ms
			if(TxRequested <= (TxComplete+NUM_TX_FRAG))
				break;
					
		//printf("%d\r\n",(int)(LPC_TIM0->TC - timer));
	}
	TxRequested++;
#endif

	TX_DESC_CTRL(pdx) = (Size-1) | TCTRL_LAST | TCTRL_INT;

	pDst = (unsigned int *)TX_DESC_PACKET(pdx);
	pSrc = (unsigned int *)Source;

	for (i = 0; i < (Size+3)>>2; i++)
		*pDst++ = *pSrc++;

	if (++pdx == NUM_TX_FRAG)
		pdx = 0;

	LPC_EMAC->TxProduceIndex = pdx;
	
}

#ifdef ETH_DBG
//  Ethernet Device Ethernet Controller Interrupt function.
void ENET_IRQHandler (void) 
{
	unsigned int  intStatus;
	while ((intStatus = (LPC_EMAC->IntStatus & LPC_EMAC->IntEnable)) != 0)
	{
		LPC_EMAC->IntClear = intStatus;
		if (intStatus & INT_TX_DONE)                       // Frame transmit completed. 
			TxComplete++;
	}
}
#endif


//#define AngelSWI 0xAB
//#define AngelSWIInsn "bkpt"
//#define AngelSWIAsm bkpt

inline int __semihost(int reason, const void *arg) {
int value;
asm volatile ("mov r0, %1; mov r1, %2; bkpt %a3; mov %0, r0"
: "=r" (value) /* Outputs */
: "r" (reason), "r" (arg), "i" (0xAB) /* Inputs */
: "r0", "r1", "r2", "r3", "ip", "lr", "memory", "cc");
//Clobbers r0 and r1, and lr if in supervisor mode
//Accordingly to page 13-77 of ARM DUI 0040D other registers
//can also be clobbered. Some memory positions may also be
//changed by a system call, so they should not be kept in
//registers. Note: we are assuming the manual is right and
//Angel is respecting the APCS.

return value;
}

#define USR_UID 0x101
int semihost_uid(char *uid) {
	uint32_t args[2]; 
	args[0] = (uint32_t)uid; 
	args[1] = 33; 
	return __semihost(USR_UID, &args); 
}

   
int GetMacAddr(void) 
{
	int i;
	char buf[48];
	i=semihost_uid((char *)&buf);
	
	//printf("semihost_uid = %d\r\n",i);
	//for(i=0;i<32;i++) printf("%d %02X\r\n",i,buf[i]);
	//for(i=0;i<32;i++) printf("%c",buf[i]);printf("\r\n");
	
	for(i=0;i<32;i++)
	{
		//ascii to hex
		if(buf[i] > 0x39)
			buf[i] -= 0x37;
		else
			buf[i] -= 0x30;
	}
	
	MyMAC[0] = (buf[20] << 4) + buf[21];
	MyMAC[1] = (buf[22] << 4) + buf[23];
	MyMAC[2] = (buf[24] << 4) + buf[25];
	MyMAC[3] = (buf[26] << 4) + buf[27];
	MyMAC[4] = (buf[28] << 4) + buf[29];
	MyMAC[5] = (buf[31] << 4) + buf[31];

	//printf("MyMAC %02X-%02X-%02X-%02X-%02X-%02X\r\n",MyMAC[0],MyMAC[1],MyMAC[2],MyMAC[3],MyMAC[4],MyMAC[5]);
	return 0;
}

