//////////////////////////////////////////////////////////////////////////////////////////////////
//
//	IRQ entries for cortex parts
//
//
// MWeb
// copyright Coridium Corp., 2014, free for any use with this notice

//
//

#include "IRQ_cortex.h"
#include "pcb.h"
#include "printf.h"

extern void doUart0Stuff();

externC void UART0_IRQHandler(void)
{
	doUart0Stuff();
}


//Returns TRUE if interrupts were disabled, FALSE if they were initially enabled.
unsigned char IntMasterEnable(void)
{
    // Enable processor interrupts.
	asm("mrs     r0, PRIMASK");
    asm("cpsie   i");
    asm("bx      lr");
	return 0; //bogus return to stop compiler warning
}

//Returns TRUE if interrupts were disabled, FALSE if they were initially enabled.
unsigned char IntMasterDisable(void)
{
    // Disable processor interrupts.
	asm("mrs     r0, PRIMASK");
    asm("cpsid   i");
    asm("bx      lr");
	return 0; //bogus return to stop compiler warning
}

unsigned disableIRQ(void)
{
  unsigned _cpsr;

  _cpsr = IntMasterDisable();
  if (_cpsr) return 0;
  else return 1;
}

unsigned restoreIRQ(unsigned oldCPSR)
{
  unsigned _cpsr;

  if(oldCPSR)
	  _cpsr = IntMasterEnable();
  else
	  _cpsr = IntMasterDisable();
	  
  if (_cpsr) return 0;
  else return 1;
}

unsigned enableIRQ(void)
{
  unsigned _cpsr;

  _cpsr = IntMasterEnable();

  if (_cpsr) return 0;
  else return 1;
} 

/////////////////////////////////////////////////////////////////////////
//
//  default and fault interrupt handlers
//

int save_regs[8];


externC void NmiSR   (void){
asm ("str lr, [r2]");
asm ("str r14, [r2]");
	printf("\r\nNmiSR    0x%04X\r\n",save_regs[0]);
	while(1);
}      


register int  save_reg asm ("r7");
externC void FaultISR(void)
{
	asm ("ldr r2,pt_save_regs");
	asm ("str lr, [r2]");
	printf("\r\nFaultISR 0x%08X SCB->CFSR=0x%08x\r\n",save_regs[0],(unsigned int) SCB->CFSR); 
	
	while(1);
}

externC void FaultMPU(void){
	asm ("ldr r2,pt_save_regs");
	asm ("str.w lr, [r2]");
	printf("\r\nFaultMPU 0x%04X\r\n",save_regs[0]);
	while(1);
}


externC void FaultBUS(void){
	asm ("ldr r2,pt_save_regs");
	asm ("str.w lr, [r2]");
	printf("\r\nFaultBUS 0x%04X\r\n",save_regs[0]);
	while(1);}

externC void FaultUSE(void)
{
	asm ("ldr r2,pt_save_regs");
	asm ("str.w lr, [r2]");
	printf("\r\nFaultUSE 0x%04X\r\n",save_regs[0]);
	while(1);
}


externC void IntDefaultHandler(void)
{
	asm ("ldr r2,pt_save_regs");
	asm ("str.w lr, [r2]");
	printf("\r\nIntDefaultHandler %08x\r\n",save_regs[0]);
	while(1){} //HANG??
}


////////////////////////////////////////////////////////
//
// keep this close by for M0

asm ("pt_save_regs:");   
asm ("    .word    save_regs");
