// MWeb
// copyright Coridium Corp., 2014, free for any use with this notice

// Choose your IP address here, MyMAC will be read from Mbed
unsigned char MyIP[4] = { 192, 168, 0, 50 };
//unsigned char MyIP[4] = { 10, 0, 0, 50 };
unsigned char MyMAC[6];

#include "pcb.h"

// forward declarations
void Check_CGI_In(void);
void SendPage(unsigned char cur_sock);
void DoUartCommand(char* line);
void SetLED(void);
int LED_state, MyVar;


int main(void)
{
	uart_linelen = uart_line[0] = 0;//clear the UART input line
	cgi_line[0] = 0;//clear the cgi input line
	
	PCB_Init(); // sets UART to 115200 baud
	
	printf("\r\nEthernet initializing\r\n"); 
	
	Init_EMAC();

	Init_Ethernet();
	
	printf("\r\nWelcome To MWeb for LPC1768\r\n");

	LPC_GPIO1->DIR |= 1 << 18; // LED 1 output
	LED_state = 1;
	SetLED();

	// Main loop, poll for Ethernet and UART
	while(1)
	{
		//check for incoming data from the usb serial port 
		if(Poll_UART0()) DoUartCommand(uart_line);
		
		Poll_Ethernet(); // check if there is a Packet received
		Check_CGI_In();  // see if there was any CGI input
		Check_TCP_Tx();  // now check if there is anything to send
	}
}


// check incoming data from an extended URL like http://192.168.0.50/?MyVar=45
void Check_CGI_In(void)
{
	unsigned int i;
	
	Read_CGI_In((char*) &cgi_line);
	if (cgi_line[0])
	{
		printf("> CGI read %s\r\n", cgi_line);
		
		// very simple example, no checking for numerical characters
		// http://192.168.0.50/?MyVar=45
		if( !STRNCMPI("?MyVar=", cgi_line, 7) )
		{
			MyVar = 0;
			i = 7;//start at first character after '='
			while(cgi_line[i])
			{
				MyVar *= 10;
				MyVar += cgi_line[i++] - 0x30; // 0x30 is ascii '0'
			}
			printf(": MyVar has been set to %d\r\n", MyVar);
		}
		
		else if( !STRNCMPI("?LED1=", cgi_line, 6) )
		{
			// very simple, 0 Off otherwise On
			if(cgi_line[6] == '0')
				LED_state = 0;
			else
				LED_state = 1;

			SetLED();
		}
			
		cgi_line[0] = 0;//clear the cgi line
	}
}

void AddLine(int* to, char *src)
{
	unsigned short i,dstpos,srcpos;
	eth_ip_tcp* tx_tcp;
	tx_tcp = (eth_ip_tcp*)GetTxBuffer();
	unsigned char *dst = tx_tcp->data;
	i = 0;
	dstpos = *to;
	srcpos = 0;
	
	//search for null terminator
	while(src[srcpos]!=0){
		dst[dstpos++] = src[srcpos++];
		i++;
	}
	if(src[srcpos]!=0){
		dst[dstpos++] = src[srcpos++];//copy \r\n
		dst[dstpos++] = src[srcpos++];
	}
	dst[dstpos] = '\0'; // keep it null terminated
	*to = dstpos;
}


// This function is called from Check_TCP_Tx to generate the web page.
// This simple example is limited to one Ethernet packet of 1460 bytes.
void SendPage(unsigned char cur_sock)
{
	int len;//, copy_length;
	unsigned char flags;

	len = 0;
	AddLine(&len, "HTTP/1.0 200 OK\r\n\r\n");
	AddLine(&len, "<html><head><script type=\"text/javascript\">\r\n");
	AddLine(&len, "function r(X){document.location.replace(\"/?\"+X);}\r\n");
	AddLine(&len, "function n(e,MyVar){if(e.keyCode==13)r(MyVar+\"=\"+document.getElementById(MyVar).value);}");
	AddLine(&len, "</script></head><body><CENTER><BIG><BIG>\r\n");
	AddLine(&len, "Click To Set LED<br><button onClick=\"r('LED1=");

	if(LED_state)
		AddLine(&len, "0')\">LED1 is ON\r\n");
	else
		AddLine(&len, "1')\">LED1 is OFF\r\n");

	AddLine(&len, "</button>\r\n");
	AddLine(&len, "<br><br>MyVar is ");
	sprintf(htm_line, "%d<br>\r\n", MyVar);
	AddLine(&len, htm_line);
	AddLine(&len, "Enter a number <input type=\"text\"id=\"MyVar\"onkeydown=\"n(event,'MyVar')\" value=\"\">");
	AddLine(&len, "</body></html>");	

	flags = TCP_CODE_ACK | TCP_CODE_FIN;
	g_sock[cur_sock].state = S_TX_SENT_FIN;//sent last packet and waiting for Ack

	send_tcp(cur_sock, len, flags);
}


// Terminal input commands you can use, modify and add to
void DoUartCommand(char* line)
{
	int i, j;
	
	printf(": %s\r\n",line);
	
	// Example: Get the A/D reading from channel 0 to 5
	// for instance enter ADC 0 or adc 5 from terminal program
	if( !STRNCMPI("ADC ", line, 4) )
	{
		i = line[4] - '0'; // - '0' is the same as - 0x30
		if ( (i < 0) || (i > 5) )
		{
			printf(": ERROR Only AD channels 0 to 5 are available\r\n");
		}
		else
		{
			Init_ADC(i); // you would actually only need to init once
			j = Get_ADC(i);
			printf(": ADC %d is 0x%03X or %d millivolts\r\n",i,j,((j*3300)+2047)>>12);//multiply by 3300, round up by 2047, divide by 4096 to get millivolts
		}
	}

	else if( STRCMPI("some command", line) )
	{
		//do something
	}
	
	else printf("Unknown Command\r\n");

	uart_linelen = uart_line[0] = 0;//clear the UART input line

}

// Set LED 1 On or Off
void SetLED(void)
{
	if(LED_state == 1)
		LPC_GPIO1->SET = 1 << 18;
	else
		LPC_GPIO1->CLR = 1 << 18;
}
