// MWeb
// copyright Coridium Corp., 2014, free for any use with this notice


#ifndef _COR_TCP_H
#define _COR_TCP_H

//TCP Flags
#define TCP_CODE_FIN		0x01
#define TCP_CODE_SYN		0x02
#define TCP_CODE_RST		0x04
#define TCP_CODE_PSH		0x08
#define TCP_CODE_ACK		0x10
#define TCP_CODE_URG		0x20
#define TCP_CODE_WAIT		0x40

#define TCP_CODE_ACKFIN		0x11
#define TCP_CODE_ACKSYN		0x12
#define TCP_CODE_ACKPSH		0x18
#define TCP_CODE_ACKFINPSH	0x19

typedef struct __attribute__((__packed__)) {
  unsigned char  dst[6];
  unsigned char  src[6];
  unsigned short eth_type;//0800 IP

  unsigned char  vhl;
  unsigned char  tos;
  unsigned short len;
  unsigned short ipid;
  unsigned short ipoffset;
  unsigned char  ttl;
  unsigned char  proto;//6 TCP
  unsigned short ipchksum;
  unsigned char  srcipaddr[4];
  unsigned char  dstipaddr[4];

  unsigned short srcport;
  unsigned short dstport;
  unsigned char  seqno[4];
  unsigned char  ackno[4];
  unsigned char  tcpoffset;//essentially tcp header length, 0x50 = 20 no options, 0x60 = 24 4 bytes options
  unsigned char  flags;
  unsigned char  wnd[2];
  unsigned short tcpchksum;
  unsigned char  urgp[2];
//  unsigned char  data[4];//option data or pointer to tcp data in which length is up to 1460 (ETH_BUFSIZE - 40)
  unsigned char  data[8];//option data or pointer to tcp data in which length is up to 1460 (ETH_BUFSIZE - 40)
  //doesn't seem to matter, but why did I make this 8 instead of 4? mar
  //maybe for the extra time stamp that I never use or allow
} eth_ip_tcp;


typedef struct __attribute__((__packed__)) {
  unsigned char state;
  //technical definition of socket is src ip, dst ip, src port, dst port
  //unsigned char srcip[4];//this is me, if it got here its obviously for me
  unsigned char retry_count;
  unsigned char seqno[4];
  unsigned char ackno[4];
  unsigned char dstmac[6];
  unsigned char dstip[4];
  unsigned short srcport;
  unsigned short dstport;
  int (*func)(unsigned char cur_sock);
  int total_to_send;//also used as total received when getting a POST
  int save_to_send;
  unsigned char htm_state;
  unsigned char save_state;
  unsigned long htm_page;
  unsigned long save_page;
  unsigned long time;
} SocketType;

//HTTP uses SocketType structure to keep some info around for transmit retries, cgi function, etc.
//in this case I allow 10 sockets, pretty much overkill
//SMTP and FTP use one of these sockets for send_tcp() but they close that socket each time, they do not support retries
//I see now I could have done this more cleanly but there are TOO many calls to send_tcp() to bother now
//If you do Not USE_HTTP_SERVER but do USE_SMTP or USE_FTP_SERVER you could save 450 bytes of RAM by defining MAX_SOCK 1

#define MAX_SOCK 10			//absolute max would be 255, 00 to FE, FF reserved for FAIL
SocketType g_sock[MAX_SOCK];

#define S_IDLE				0
#define S_RX_SYNC_REQ		1	//received new Sync request, set up to send AckSync
#define S_TX_ASYNC			2	//sent AckSync
#define S_RX_ACK_ASYNC		3	//received Ack of AckSync, set up to wait 10 seconds for a GET or POST request
#define S_GOT_REQUEST		4	//received a GET or POST request, set up to send the first packet
#define S_TX_SEND_DATA		5	//set up to send subsequent packets
#define S_TX_SENT_DATA		6	//sent a packet and waiting for Ack, used for retry of missed packets
#define S_TX_SENT_FIN		7	//sent the last packet and waiting for Ack, used for retry of missed packets
#define S_WAIT_ACKFIN		8	//set up to wait for AckFin
#define S_TX_ACK_FIN		9	//set up to send Ack and close the socket
#define SOCK_POST0			10	//this is the start of a POST, may or may not contain data
#define SOCK_POST1			11	//this is the start of a POST boundary, read through the boundary and save it
#define SOCK_POST2			12	//this is the start of a POST data
#define SOCK_POST3			13	//this is the end boundary, separate state in case it spans 2 packets
		 

//HTTP Server

#define HTTP_RETRY_DELAY 3000000
#define HTTP_RETRY_MAX 3

int g_http_rx_cnt;
void Check_TCP_Tx(void);// Check if TX FIFO is empty then if any sockets have timed out and need to TX, only doing one sock op at a time
void Handle_HTTP();

#define CGI_BUFFER_SIZE 256
char g_cgibuf[CGI_BUFFER_SIZE];

void Read_CGI_In(char * cgi_string);// Copy g_cgibuf to a user string buffer

#define HTTP_OUTBUFFSIZE 0xB00 //2816
unsigned char g_xOutBuf[HTTP_OUTBUFFSIZE];// bounds controlled in printCH
unsigned short g_OutBufPtr;
void tcpPutch(char ch);
void send_tcp(unsigned char cur_sock, unsigned short tcp_data_len, unsigned char flags);

//int tcpGetch(void);



#endif
