// MWeb
// copyright Coridium Corp., 2014, free for any use with this notice


#include "pcb.h"

extern int printf(const char *format, ...);

void PCB_Init (void) {
	//volatile int i;

	//Switch CPU to PLL clock
	LPC_SC->PLL0CON = 0;				// Disable PLL, disconnected
	LPC_SC->PLL0FEED = 0xaa;
	LPC_SC->PLL0FEED = 0x55;

	//USE External Main Oscillator
	LPC_SC->SCS = 0x20;				// Enable main OSC
	while( !(LPC_SC->SCS & 0x40) );	// Wait until main OSC is usable
	LPC_SC->CLKSRCSEL = 1;			//External Main Oscillator

	LPC_SC->PLL0CFG  = PLL_MValue | (PLL_NValue << 16); //Set Multiplier and Pre-Divider
	LPC_SC->PLL0FEED = 0xaa;
	LPC_SC->PLL0FEED = 0x55;

	LPC_SC->PLL0CON = PLLCON_PLLE;				// Enable PLL, disconnected
	LPC_SC->PLL0FEED = 0xaa;
	LPC_SC->PLL0FEED = 0x55;

	LPC_SC->CCLKCFG = CCLKSEL;	// Set clock divider

	while ( (LPC_SC->PLL0STAT & PLLSTAT_LOCK) == 0 ) ;	// Wait for lock bit status

	LPC_SC->PLL0CON = PLLCON_PLLE | PLLCON_PLLC; // enable and connect
	LPC_SC->PLL0FEED = 0xaa;
	LPC_SC->PLL0FEED = 0x55;

	while ( ((LPC_SC->PLL0STAT & PLLSTAT_CONNECT) == 0) );	// Wait for connect bit status

	LPC_PINCON->PINSEL0 = 0x00000050;   // RxD0 and TxD0
	
	// Setup UART0
	LPC_UART0->IER = 0x00;              // disable all interrupts
	NVIC->ISER[0] = 1 << UART0_IRQn;    // enable interrupt UART0
	LPC_UART0->IIR;                     // clear interrupt ID
	LPC_UART0->RBR;                     // clear receive register
	LPC_UART0->LSR;                     // clear line status register
	LPC_UART0->LCR |= UART_LCR_DLAB_EN; // select divisor latches 
	LPC_UART0->DLL = BAUD_115200;       // set for baud low byte
	LPC_UART0->DLM = 0;    				// set for baud high byte
	LPC_UART0->LCR = ((UART_CFG_DATABIT_8 | UART_CFG_STOPBIT_1 | UART_CFG_PARITY_NONE) & ~UART_LCR_DLAB_EN);
	LPC_UART0->FCR = UART_FCR_FIFO_EN | UART_CFG_FIFOTRG_8;
	LPC_UART0->IER = UART_INTCFG_RBR;   // Enable UART0 RX interrupt
	
	// Setup Real Time Clock         
	LPC_RTC->CIIR=0;  	//no increment interrupts
	LPC_RTC->AMR=0x58;	//use YEAR, DOY, HOUR, MIN, SEC
	LPC_RTC->CCR = 0x11;//enable and use 32kHz

	// Setup Timer0, free running microsecond counter
	LPC_TIM0->PR  = (SystemCoreClock/4/1000000)-1;			//1 us prescaler
	LPC_TIM0->TC  = 0;
	LPC_TIM0->TCR = 1;          // Timer0 Enable

	enableIRQ();
}

void Init_ADC(int channel)
{
	LPC_SC->PCONP |= (1 << 12);//enable power for AD

	//NOTE many people would use a case here but I've always prefered if else if
	if (channel == 0){
		LPC_PINCON->PINSEL1 &= ~(3 << 14);//clear existing value
		LPC_PINCON->PINSEL1 |=   1 << 14; //P0.23 is AD0.0
		LPC_PINCON->PINMODE1 &= ~(3 << 14);//clear existing value
		LPC_PINCON->PINMODE1 |=   2 << 14; //P0.23 has neither pull-up nor pull-down
	} 
	else if (channel == 1){
		LPC_PINCON->PINSEL1 &= ~(3 << 16);
		LPC_PINCON->PINSEL1 |=   1 << 16; //P0.24 is AD0.1
		LPC_PINCON->PINMODE1 &= ~(3 << 16);
		LPC_PINCON->PINMODE1 |=   2 << 16; //P0.24 has neither pull-up nor pull-down
	} 
	else if (channel == 2){
		LPC_PINCON->PINSEL1 &= ~(3 << 18);
		LPC_PINCON->PINSEL1 |=   1 << 18; //P0.25 is AD0.2
		LPC_PINCON->PINMODE1 &= ~(3 << 18);
		LPC_PINCON->PINMODE1 |=   2 << 18; //P0.25 has neither pull-up nor pull-down
	} 
	else if (channel == 3){
		LPC_PINCON->PINSEL1 &= ~(3 << 20);
		LPC_PINCON->PINSEL1 |=   1 << 20; //P0.26 is AD0.3
		LPC_PINCON->PINMODE1 &= ~(3 << 20);
		LPC_PINCON->PINMODE1 |=   2 << 20; //P0.26 has neither pull-up nor pull-down
		//P0.26 is also AOUT
		//So you can't use ADC 3 and DAC at the same time
	} 
	else if (channel == 4){
		LPC_PINCON->PINSEL3 |=   3 << 28; //P1.30 is AD0.4
		LPC_PINCON->PINMODE3 &= ~(3 << 28);
		LPC_PINCON->PINMODE3 |=   2 << 28; //P1.30 has neither pull-up nor pull-down
	} 
	else if (channel == 5){
		LPC_PINCON->PINSEL3 |=   3 << 30; //P1.31 is AD0.5
		LPC_PINCON->PINMODE3 &= ~(3 << 30);
		LPC_PINCON->PINMODE3 |=   2 << 30; //P1.31 has neither pull-up nor pull-down
	} 

	// These 2 pins are used for UART0 but if they weren't here's how you would do it
	//else if (channel == 6){
	//	LPC_PINCON->PINSEL0 &= ~(3 << 6);
	//	LPC_PINCON->PINSEL0 |=   1 << 6; //P0.3 is AD0.6
	//	LPC_PINCON->PINMODE0 &= ~(3 << 6);
	//	LPC_PINCON->PINMODE0 |=   2 << 6; //
	//} 
	//else if (channel == 7){
	//	LPC_PINCON->PINSEL0 &= ~(3 << 4);
	//	LPC_PINCON->PINSEL0 |=   1 << 4; //P0.2 is AD0.7
	//	LPC_PINCON->PINMODE0 &= ~(3 << 4);
	//	LPC_PINCON->PINMODE0 |=   2 << 4; //
	//} 
}

//Read ADC, return raw 12 bit AD conversion
int Get_ADC(int channel)
{
	unsigned long timer;
	int ad_in;
	
	if( (channel < 0) || (channel > 5) )
		return -1;

	LPC_ADC->CR =	 // Start a conversion
	(1 << 24) | // START = 001 Start conversion now
	(1 << 21) | // PDN = 1 turns it on
	(1 << 8 ) | // CLKDIV = 0x02,  PCLK is 25 MHz so ADCLK is 12.5 MHz, must be 13 MHz or less for LPC17xx
	(1 << channel); // channel

	timer = LPC_TIM0->TC; // time out timer for a little extra safety
	
	while ( !(LPC_ADC->GDR & 0x80000000) && ( (LPC_TIM0->TC - timer) < 50) ){} //assume success
	ad_in = (((int)LPC_ADC->GDR >> 4) & 0xFFF); //raw 12 bit AD conversion
	
	//timer = LPC_TIM0->TC - timer; //usually takes 6 microseconds
	//printf("AD conversion %03X took %d us\r\n", ad_in, (int)timer);
	
	return ad_in;
}

// Set 10-bit digital to analog converter
void Set_DAC(int value)
{
	LPC_PINCON->PINSEL1 &= ~(3 << 20);
	LPC_PINCON->PINSEL1 |=   2 << 20; //P0.26 is AOUT
	LPC_PINCON->PINMODE1 &= ~(3 << 20);
	LPC_PINCON->PINMODE1 |=   2 << 20; //P0.26 has neither pull-up nor pull-down
	//P0.26 is also ADC 3
	//So you can't use ADC 3 and DAC at the same time
	LPC_DAC->DACR = value << 6;
}

// Interrupt routine called by UART0_IRQHandler
void doUart0Stuff() {

	char iid, ch;
	int temp;

	iid = LPC_UART0->IIR & 0xe;  //UART_INTSTAT_PEND             // ignore INTERRUPTS PENDING

    // identify & process the highest priority interrupt
	if ((iid == IIR_RDA) || (iid == IIR_CTI)) {

          // calc next insert index & store character
          temp = (uart0_rx_insert_idx + 1) & (UART_RX_BUFFER_SIZE-1);
          ch = LPC_UART0->RBR;
          uart0_rx_buffer[uart0_rx_insert_idx] = ch;

          // check for more room in queue
          if (temp != uart0_rx_extract_idx) uart0_rx_insert_idx = temp; // update insert index
  	} else if (iid == IIR_THRE) {               // Transmit Holding Register Empty
	} else {                          // error conditions -- read status and character buffer and exit
        LPC_UART0->LSR;
        LPC_UART0->RBR;
	}
  #ifdef VICVectAddr		// doesn't exist in Cortex
	VICVectAddr = 0x00000000;             // clear this interrupt from the VIC
  #endif
}

// print a character to UART0
int putchar(int ch) {
	tcpPutch(ch);
	while ((LPC_UART0->LSR & UART_LS_TX_EMPTY_ALL) == 0);
  	LPC_UART0->THR = (char)ch;
	return ch;
}

// read a character from UART0
int getchar(){
	char ch;
  
	if (uart0_rx_insert_idx == uart0_rx_extract_idx) // check if character is available
		return -1;
	ch = uart0_rx_buffer[(uart0_rx_extract_idx)++]; // get character, bump pointer
	uart0_rx_extract_idx &= (UART_RX_BUFFER_SIZE-1); // limit the pointer
	return ch;
}

// poll Uart0 , if a newline is found return the line length, else return 0
int Poll_UART0(void)
{
	char c;
	int i;
		
	i = getchar();
	if (i != -1){
		c = i & 0xFF;//safety

		// end on either CR or LF
		// if a LF follows a CR it simply returns 0, as will multiple blank lines
		if ((c == 0xD) || (c == 0xA))
			return uart_linelen;

		uart_line[uart_linelen++] = c;
		if(uart_linelen >= MAX_LINE_LENGTH)
			uart_linelen--;
	}
	uart_line[uart_linelen] = '\0';//keep it null terminated
	return 0;
}	


unsigned char STRCMPI(char *s1, char *s2)
{
	char c1,c2;
	while(*s1 || *s2){
		c1=*s1++;
		if(c1>='a' && c1<='z')
			c1 &= 0x5F;
		c2=*s2++;
		if(c2>='a' && c2<='z')
			c2 &= 0x5F;
		if(c1 != c2)
			return 1;
	}
	return 0;
}

unsigned char STRCMP(char *s1, char *s2)
{
	while(*s1 || *s2){
		if(*s1++ != *s2++)
			return 1;
	}
	return 0;
}

void STRCPY(char *dst, char *src)
{
	while (*src) *dst++ = *src++;
	*dst = '\0';
}

unsigned char STRNCMPI(char *s1, char *s2, unsigned char N)
{
	unsigned char i;
	i = N;
	char c1,c2;
	while(*s1 && *s2){
		c1=*s1++;
		if(c1>='a' && c1<='z')
			c1 &= 0x5F;
		c2=*s2++;
		if(c2>='a' && c2<='z')
			c2 &= 0x5F;
		if(c1 != c2)
			return 1;
		i--;
		if(i==0)
			break;
	}
	return 0;
}

unsigned char MEMCMP(char *s1, char *s2, unsigned char N)
{
	unsigned char i;
	i = 0;
	while(i<N){
		if(*s1++ != *s2++)
			return i+1;
		i++;
	}
	return 0;
}

//void MEMSET(char *dst, char val, int len)
void * MEMSET(void *m, char c, int n)
{
  char *s = (char *) m;

  while (n-- != 0)
    {
      *s++ = (char) c;
    }

  return m;
}

//void MEMCPY(char *dst0, char *src0, int len0)
void * MEMCPY(void *dst0, void *src0, int len0)
{
  char *dst = (char *) dst0;
  char *src = (char *) src0;

  void * save = dst0;

  while (len0--)
    {
      *dst++ = *src++;
    }

  return save;
}

void STRCAT(char *dst, char *src)
{
	while (*dst)dst++;
	while (*src) *dst++ = *src++;
	*dst = '\0';
}

int STRLEN (char *s){
  char *start = s;

  while (*s)
    s++;

  return s - start;
}

void wait(int ms)
{
	unsigned long timer = LPC_TIM0->TC;
	while( (LPC_TIM0->TC - timer) < ms*1000){}
}

////////////////////////////////////////////////////////////////////////////////////
//
//   hacks for Yagarto 4.6.0  -- the list is growing...

#if (__GNUC__ == 4) && (__GNUC_MINOR__ == 6) //&& (__GNUC_PATCHLEVEL__ == 0)

  // this is a to disable all the unwind stuff generated in 4.6.0
  char __aeabi_unwind_cpp_pr0[0];

#endif



