#ifndef _CORIDIUM_PCB_H
#define _CORIDIUM_PCB_H

////////////////////////////////////////////////////////////////////////////
//
// MWeb
// copyright Coridium Corp., 2014, free for any use with this notice


//
//	this file should be included in all sources
//


#include "LPC175x6x.h"
#include "IRQ_cortex.h"
#include "EMAC.h"
#include "printf.h"
#include "cor_ethernet.h"
#include "eth_arp.h"
#include "eth_ip_icmp.h"
#include "eth_ip_tcp.h"

// the include files are slightly different
#define AUXEN RTC_AUXEN
#define AUX   RTC_AUX

#define IIR_RDA         (0x02<<1)
#define IIR_CTI         (0x06<<1)
#define IIR_THRE        (0x01<<1)
#define LSR_THRE        (0x01<<5)
#define UART_LS_TX_EMPTY_ALL  ((uint8_t)(1<<6)) /*Transmitter THR and TSR all empty*/

#define CLK_FREQ_KHZ   12000
//Setup Main Oscillator 12 MHz for 100 MHz CPU Clock and 25 MHz Peripheralbus Clock
#define PLL_CKIN   12000000//12 MHz , set CLKSRCSEL = 1
#define PLL_MValue 49      //M + 1 = 50
#define PLL_NValue 2       //N + 1 = 3
#define PLL_CLK  400000000 //12 MHz * 2 * 50 / 3 = 400 MHz, must be 275 to 550
#define CCLKSEL    3       //CCLKSEL + 1 = 4, CCLKSEL must be 0 or odd value
#define CCLK      100000000//400 MHz / 4 = 100 MHz
#define PCLK      25000000 //using PCLK is 1/4 CLK for everything but UART1

#define Clk_100MHz 100000000
#define SystemCoreClock Clk_100MHz

// PCLK = 1/4 CCLK
#define BAUD_19200  81      //DIV = ( PCLK / 16 ) / 19200  (actaul 81.3) using PCLK is 1/4 CLK for UART0
#define BAUD_115200 13      //actaul 13.5
//115200 is 13.5
#define UART_LCR_DLAB_EN    (1<<7)  // DivisorLatchesAccess enable
#define UART_CFG_DATABIT_8  (3<<0)  // UART 8 bit data mode 
#define UART_CFG_STOPBIT_1   (0)    // UART 1 Stop Bits Select 
#define UART_CFG_PARITY_NONE (0)    // No parity 
#define UART_FCR_FIFO_EN     (1<<0) // FIFO enable 
#define UART_CFG_FIFOTRG_8   (2<<6) // UART FIFO trigger level 2: 8 char 
#define UART_INTCFG_RBR      (1<<0) // RBR Interrupt enable

#define PLLSTAT_LOCK  (1 << 26)     // PLL Lock Status Bit
#define PLLSTAT_CONNECT (1 << 25)   // PLL Connect Status Bit
#define PLLSTAT_ENABLE (1 << 24)    // PLL Enable Status Bit
#define PLLCON_PLLE   (1 << 0)      // PLL Enable
#define PLLCON_PLLC   (1 << 1)      // PLL Connect


//These two are Way bigger than necessary, they could easily be 32 or at most 64
#define UART_RX_BUFFER_SIZE	256		// UART buffers must be powers of 2
#define MAX_LINE_LENGTH 128

volatile int 	uart0_rx_insert_idx, uart0_rx_extract_idx, uart0_tx_insert_idx, uart0_tx_extract_idx;
volatile char uart0_rx_buffer[UART_RX_BUFFER_SIZE];

char cgi_line[MAX_LINE_LENGTH];				
char htm_line[MAX_LINE_LENGTH]; // I could probably re-use cgi_line but no need to be stingy here
char uart_line[MAX_LINE_LENGTH];				
int uart_linelen;

void PCB_Init (void);
void Init_ADC(int channel);
int Get_ADC(int channel);

int putchar(int ch) ;// print a character to UART0
int getchar();// read a character from UART0
void doUart0Stuff();
int Poll_UART0(void);

unsigned char STRCMPI(char *s1, char *s2);
unsigned char STRCMP(char *s1, char *s2);
void STRCPY(char *dst, char *src);
unsigned char STRNCMPI(char *s1, char *s2, unsigned char N);
unsigned char MEMCMP(char *s1, char *s2, unsigned char N);//STRNCMP s/b same as MEMCMP
void * MEMSET(void *m, char c, int n);
void * MEMCPY(void *dst0, void *src0, int len0);
void STRCAT(char *dst, char *src);
int STRLEN (char *s);
void wait(int ms);


////////////////////////////////////////////////////////////////////////////////////
//
//   hacks for Yagarto 4.6.0  -- the list is growing...

#if (__GNUC__ == 4) && (__GNUC_MINOR__ == 6) // && (__GNUC_PATCHLEVEL__ == 0)
 #define main	main_embedded		// otherwise the linker sticks it in a strange place
#endif

// the following defined in coridium_pcb.c -- seems to make linker _sbrk and _unwind errors go away
//   char __aeabi_unwind_cpp_pr0[0];
//   

#endif