
// UART stuff

// modfications by Coridium Corp 2011 

//#define UART_NO_INTERRUPT	// use this during debug

#include "coridium_pcb.h"
#include "uart.h"

////////////////////////////////////////////////////////////////////////////////
//
// buffers and pointers for UARTs
//
// finally define them here


volatile int 	uart0_rx_insert_idx, uart0_rx_extract_idx, uart0_tx_insert_idx, uart0_tx_extract_idx;
volatile char uart0_rx_buffer[UART_RX_BUFFER_SIZE];
volatile char uart0_tx_buffer[UART_TX_BUFFER_SIZE];
#ifdef LPC_UART1
volatile int 	uart1_rx_insert_idx, uart1_rx_extract_idx, uart1_tx_insert_idx, uart1_tx_extract_idx;
volatile char uart1_rx_buffer[UART_RX_BUFFER_SIZE];
volatile char uart1_tx_buffer[UART_TX_BUFFER_SIZE];
#endif  			
#ifdef LPC_UART2
volatile int 	uart2_rx_insert_idx, uart2_rx_extract_idx, uart2_tx_insert_idx, uart2_tx_extract_idx;
volatile char uart2_rx_buffer[UART_RX_BUFFER_SIZE];
volatile char uart2_tx_buffer[UART_TX_BUFFER_SIZE];
#endif  			
#ifdef LPC_UART3
volatile int 	uart3_rx_insert_idx, uart3_rx_extract_idx, uart3_tx_insert_idx, uart3_tx_extract_idx;
volatile char uart3_rx_buffer[UART_RX_BUFFER_SIZE];
volatile char uart3_tx_buffer[UART_TX_BUFFER_SIZE];
#endif  			


/*
  * @brief       Initializes the UARTx peripheral.
  * @param    UARTx   UART peripheral selected, should be LPC_UART0, LPC_UART1.
  * @return     None
  */
void UART_init(int uartNUM, int baud) {

	LPC_UART_TypeDef      *UARTX;
	int clk_div;					// set to the PCLK value of the UART
	
	switch(uartNUM) {
		case 0:
			UARTX = (LPC_UART_TypeDef *)LPC_UART0;
			uart0_rx_insert_idx = uart0_rx_extract_idx = uart0_tx_insert_idx = uart0_tx_extract_idx = 0;
  #ifdef LPC175x				// TXD0 = P0.2  RXD0 = P03
			LPC_PINCON->PINSEL0 = (LPC_PINCON->PINSEL0 & ~0xF0) | 0x00000050;       // RxD0 and TxD0
			clk_div = ((LPC_SC->PCLKSEL0>>6)&3)==0 ? 4 : (((LPC_SC->PCLKSEL0>>6)&3)==3 ? 8 : (LPC_SC->PCLKSEL0>>6)&3) ;	
  #elif defined LPC178x				// TXD0 = P0.2  RXD0 = P03
  			LPC_IOCON->P0_2 = 1;
  			LPC_IOCON->P0_3 = 1;
			clk_div = LPC_SC->PCLKSEL & 0x1f;	
  #elif defined LPC12xx			
			LPC_IOCON->PIO0_2 = 2;		// enable RXD0
			LPC_IOCON->PIO0_1 = 2;		// enable TXD0
			NVIC->ISER[0] = 1 << UART0_IRQn;// enable interrupt UART1
			clk_div = LPC_SYSCON->UART0CLKDIV;
  #elif defined LPC111x
			LPC_IOCON->PIO1_6 = 1;		// enable RXD0
			LPC_IOCON->PIO1_7 = 1;		// enable TXD0
			NVIC->ISER[0] = 1 << UART0_IRQn;// enable interrupt UART1
			clk_div = LPC_SYSCON->UARTCLKDIV & 0xff;	
			LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12); //turn on the UART clock
  #elif defined LPC21xx
			PCB_PINSEL0 = (PCB_PINSEL0 & ~0xF) | 0x5;  // enable TXD0, RXD0
			clk_div = (SCB_APBDIV&3)==0 ? 4 : (SCB_APBDIV&3);
  #else
    #error   Unknown CPU type in UART_Init
  #endif  			
			break;
 #ifdef LPC_UART1			
		case 1:
			UARTX = (LPC_UART_TypeDef *)LPC_UART1; //M3 regs are at 0x4000C000 and 0x40010000
			uart1_rx_insert_idx = uart1_rx_extract_idx = uart1_tx_insert_idx = uart1_tx_extract_idx = 0;
  #ifdef LPC175x				// TXD1 = P2.0  RXD1 = P2.1
			LPC_PINCON->PINSEL4 = (LPC_PINCON->PINSEL4 & ~0xF) | 0x0000000A;       // RxD1 and TxD1
			clk_div = ((LPC_SC->PCLKSEL0>>8)&3)==0 ? 4 : (((LPC_SC->PCLKSEL0>>8)&3)==3 ? 8 : (LPC_SC->PCLKSEL0>>8)&3) ;	
			NVIC->ISER[0] = 1 << UART1_IRQn;// enable interrupt UART1
  #elif defined LPC178x				
			clk_div = LPC_SC->PCLKSEL & 0x1f;	
     #warning -- need uart def for 177x UART1
  #elif defined LPC12xx			
			LPC_IOCON->PIO2_12 = 3;		// enable RXD1
			LPC_IOCON->PIO2_13 = 3;		// enable TXD1
			clk_div = LPC_SYSCON->UART1CLKDIV;
			NVIC->ISER[0] = 1 << UART1_IRQn;// enable interrupt UART1
  #elif defined LPC21xx
			PCB_PINSEL0 = (PCB_PINSEL0 & ~0xF0000) | 0x50000;  // enable TXD1, RXD1
			clk_div = (SCB_APBDIV&3)==0 ? 4 : (SCB_APBDIV&3);
  #else
    #error   Unknown CPU type in UART_Init
  #endif  			
			break;
 #endif			
 #ifdef LPC_UART2	
  #ifdef LPC175x				// TXD2 = P2.8  RXD2 = P2.9
		case 2:
			LPC_SC->PCONP |= 1<<24;				 // turn it on
			UARTX = (LPC_UART_TypeDef *)LPC_UART2; //M3 regs are at 0x4000C000 and 0x40010000
			uart2_rx_insert_idx = uart2_rx_extract_idx = uart2_tx_insert_idx = uart2_tx_extract_idx = 0;
			LPC_PINCON->PINSEL4 = (LPC_PINCON->PINSEL4 & ~0xF0000) | 0x000A0000;       // on P2 pin 8 and 9
			clk_div = ((LPC_SC->PCLKSEL1>>16)&3)==0 ? 4 : (((LPC_SC->PCLKSEL1>>16)&3)==3 ? 8 : (LPC_SC->PCLKSEL1>>16)&3) ;	
			NVIC->ISER[0] = 1 << UART2_IRQn;// enable interrupt UART1
			break;
  #elif defined LPC178x				
			clk_div = LPC_SC->PCLKSEL ;	
     #warning -- need uart def for 177x UART1
  #else
    #error   Unknown CPU type in UART_Init
  #endif  			
 #endif			
 #ifdef LPC_UART3	
  #ifdef LPC175x				// TXD3 = P4.28  RXD3 = P4.29
		case 3:
			LPC_SC->PCONP |= 1<<25;				 // turn it on
			UARTX = (LPC_UART_TypeDef *)LPC_UART3; //M3 regs are at 0x4000C000 and 0x40010000
			uart3_rx_insert_idx = uart3_rx_extract_idx = uart3_tx_insert_idx = uart3_tx_extract_idx = 0;
			LPC_PINCON->PINSEL9 = (LPC_PINCON->PINSEL9 & ~0x0F000000) | 0x0F000000;       // on P4 pin 28 and 29
			clk_div = ((LPC_SC->PCLKSEL1>>18)&3)==0 ? 4 : (((LPC_SC->PCLKSEL1>>18)&3)==3 ? 8 : (LPC_SC->PCLKSEL1>>18)&3) ;	
			NVIC->ISER[0] = 1 << UART3_IRQn;// enable interrupt UART1
			break;
  #elif defined LPC178x				
			clk_div = LPC_SC->PCLKSEL ;	
     #warning -- need uart def for 177x UART1
  #else
    #error   Unknown CPU type in UART_Init
  #endif  			
 #endif			
		default:
			return;
	}
	UARTX->IER = 0x00;                       // disable all interrupts

	baud = ((SystemCoreClock / clk_div / baud) + 7) >> 4;		// add 7 to round the result/16
	
	switch(uartNUM) {
 #ifdef NVIC		
		case 0:
			NVIC->ISER[0] = 1 << UART0_IRQn;// enable interrupt UART0
			break;
  #ifdef LPC_UART1			
		case 1:
			NVIC->ISER[0] = 1 << UART1_IRQn;// enable interrupt UART1
			break;
  #endif			
  #ifdef LPC_UART2			
		case 2:
			NVIC->ISER[0] = 1 << UART2_IRQn;// enable interrupt UART2
			break;
  #endif			
  #ifdef LPC_UART3			
		case 3:
			NVIC->ISER[0] = 1 << UART3_IRQn;// enable interrupt UART3
			break;
  #endif			
  #ifdef LPC_UART4			
		case 4:
			NVIC->ISER[1] = 1 << (UART4_IRQn - 32);// enable interrupt UART3
			break;
  #endif			
 #else
		case 0:
			VICVectCntl0 = VIC_ENABLE | VIC_UART0;	// choice of interupt 10 is arbitrary
			VICVectAddr0 = (unsigned int)UART0_IRQHandler;	// address of the ISR
			VICIntEnable |= 1 << VIC_UART0;    		// UART0 interrupt enabled
			break;
		case 1: 
			VICVectCntl3 = VIC_ENABLE | VIC_UART1;	// choice of interupt 11 is arbitrary
			VICVectAddr3 = (unsigned int)UART1_IRQHandler;	// address of the ISR
			VICIntEnable |= 1 << VIC_UART1;    		// UART1 interrupt enabled
			break;
 #endif			
		default:
			break;
	}
	
	UARTX->IIR;                              // clear interrupt ID
	UARTX->RBR;                              // clear receive register
	UARTX->LSR;                              // clear line status register

	// set the baudrate
	UARTX->LCR |= UART_LCR_DLAB_EN;           // select divisor latches 
	UARTX->DLL = baud;        				// set for baud low byte
	UARTX->DLM = baud >> 8;    				// set for baud high byte
	
	// set the number of characters and other
	// user specified operating parameters
	
	UARTX->LCR = ((UART_CFG_DATABIT_8 | UART_CFG_STOPBIT_1 | UART_CFG_PARITY_NONE) & ~UART_LCR_DLAB_EN);
	UARTX->FCR = UART_FCR_FIFO_EN | UART_CFG_FIFOTRG_8;
	
  #ifdef TX_USE_BUFFER
	// enable receiver and transmit interrupts
	UARTX->IER = UART_INTCFG_RBR | UART_INTCFG_THRE;
  #else	
	// enable receiver interrupts
	UARTX->IER = UART_INTCFG_RBR;
  #endif

  
}

//////////////////////////////////////////////////////////////////////////////
//
//	all uart interrupt come to this code with pointers set to
//	the proper set of UART registers and buffers and buffer pointers
//


void doUartStuff(LPC_UART_TypeDef *LPC_UARTX, char uart_tx_buffer[], int *uart_tx_insert_idx,int *uart_tx_extract_idx, 
											  char uart_rx_buffer[], int *uart_rx_insert_idx,int *uart_rx_extract_idx) {

	char iid, ch;
	int temp;
  #ifdef TX_USE_BUFFER
	char uartTXFIFOcnt;
  #endif

	iid = LPC_UARTX->IIR & 0xe;  //UART_INTSTAT_PEND             // ignore INTERRUPTS PENDING

    // identify & process the highest priority interrupt
	if ((iid == IIR_RDA) || (iid == IIR_CTI)) {

          // calc next insert index & store character
          temp = (*uart_rx_insert_idx + 1) & (UART_RX_BUFFER_SIZE-1);
          ch = LPC_UARTX->RBR;
          uart_rx_buffer[*uart_rx_insert_idx] = ch;

          // check for more room in queue
          if (temp != *uart_rx_extract_idx) *uart_rx_insert_idx = temp; // update insert index
  	} else if (iid == IIR_THRE) {               // Transmit Holding Register Empty
 #ifdef TX_USE_BUFFER
		uartTXFIFOcnt = 12;				// seems like 16 breaks it
        do {
          // check if more data to send
			if (*uart_tx_insert_idx == *uart_tx_extract_idx) break; // no data left to send
			
           	LPC_UARTX->THR = uart_tx_buffer[*uart_tx_extract_idx];
           	*uart_tx_extract_idx = (*uart_tx_extract_idx+1) & (UART_TX_BUFFER_SIZE-1);
		} while (uartTXFIFOcnt--) ;
        LPC_UARTX->IIR;
 #endif
	} else {                          // error conditions -- read status and character buffer and exit
        LPC_UARTX->LSR;
        LPC_UARTX->RBR;
	}
  #ifdef VICVectAddr		// doesn't exist in Cortex
	VICVectAddr = 0x00000000;             // clear this interrupt from the VIC
  #endif
}



/////////////////////////////////////////////////////
//
//	general interfaces to send and receive characters from the UARTs
//

int uartGetch(LPC_UART_TypeDef *LPC_UARTX,volatile char uart_rx_buffer[],volatile int *uart_rx_insert_idx,volatile int *uart_rx_extract_idx)
{
 #ifdef UART_NOBUFFER		// use this during bring up when interrupts aren't working yet
	if (LPC_UARTX->LSR & 1) return LPC_UARTX->RBR;
	else return -1;
 #else  
	char ch;
  
	if (*uart_rx_insert_idx == *uart_rx_extract_idx) // check if character is available
		return -1;
	ch = uart_rx_buffer[(*uart_rx_extract_idx)++]; // get character, bump pointer
	*uart_rx_extract_idx &= (UART_RX_BUFFER_SIZE-1); // limit the pointer
	return ch;
 #endif  
}



int uartPutch(LPC_UART_TypeDef *LPC_UARTX, char ch)		// FIXME XOFF has to go ahead of the transmit buffer, if that ever gets enabled
{
	while ((LPC_UARTX->LSR & UART_LS_TX_EMPTY_ALL) == 0);
  	LPC_UARTX->THR = (char)ch;

  	return ch;
}

// use the following until TX interrupts and buffer enabled

int uartPuts(LPC_UART_TypeDef *LPC_UARTX, char *string)
{
	int uartTXFIFOcnt;
  	char ch;

  	uartTXFIFOcnt = 0;			// can't read the FIFO status wait for empty and then send 16 bytes
  	
  	while ((ch = *string++) != 0) {
		if (uartTXFIFOcnt == 0) {
			while ((LPC_UARTX->LSR & UART_LS_TX_EMPTY_ALL) == 0);
	  		uartTXFIFOcnt = 16;			// can't read this reliably
  		}
	  	LPC_UARTX->THR = (char)ch;
	  	uartTXFIFOcnt--;
	}
	return 0;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////
//
//	standard C like access to the UARTs as streams  -- getchar, getc, putchar, putc
//

// we could mimic C more by adding -- and defining UARTnum as a FILE

// struct __FILE { int handle; /* Add whatever you need here */ };
// typedef FILE __FILE;

int getchar(void) {
  #ifdef UART_NO_INTERRUPT
	if (LPC_UART0->LSR & 1) return LPC_UART0->RBR;
	else return -1;
  #else
	return uartGetch((LPC_UART_TypeDef *)LPC_UART0, uart0_rx_buffer, &uart0_rx_insert_idx, &uart0_rx_extract_idx);
  #endif
}

int getc(int UARTnum) {
	if (UARTnum == 0) return uartGetch((LPC_UART_TypeDef *)LPC_UART0, uart0_rx_buffer, &uart0_rx_insert_idx, &uart0_rx_extract_idx);
  #ifdef LPC_UART1
	if (UARTnum == 1) return uartGetch((LPC_UART_TypeDef *)LPC_UART1, uart1_rx_buffer, &uart1_rx_insert_idx, &uart1_rx_extract_idx);
  #endif	
  #ifdef LPC_UART2
	if (UARTnum == 2) return uartGetch(LPC_UART2, uart2_rx_buffer, &uart2_rx_insert_idx, &uart2_rx_extract_idx);
  #endif	
  #ifdef LPC_UART3
	if (UARTnum == 3) return uartGetch(LPC_UART3, uart3_rx_buffer, &uart3_rx_insert_idx, &uart3_rx_extract_idx);
  #endif	
  #ifdef LPC_UART4
 #warning need lots o stuff for UART4
  #endif	
	return -1;
}

#ifndef CR
 #define CR 0xd
 #define LF 0xa
#endif

int fgets_last_ch;  // handle  LF same as CR or CR/LF

char * fgets (char * string, int max_count, int UARTnum) {
	char * save_string;
	int ch;
	
	save_string = string;
	
	do {
		do {
			ch = getc(UARTnum);
		} while (ch == -1);
		if (ch == LF) {
			if (fgets_last_ch == CR) continue;
			*string=0;
			break;
		}
		if (ch == CR) {
			*string=0;
			break;
		}
		*string++ = ch;
		if(ch == 0) break;  	// unlikely, but could happen
	} while (--max_count);
	fgets_last_ch = ch;
	return save_string;
}

char * gets(char * string) {
	return fgets(string, 255, 0);
}

int putchar(int ch) {
	echo_char(ch);				// in ARMweb this redirects IO to web page, when debugging sends to both
	return uartPutch((LPC_UART_TypeDef *)LPC_UART0, ch);
}

int putc(int ch, int UARTnum) {
	if (UARTnum == 0) return uartPutch((LPC_UART_TypeDef *)LPC_UART0, ch);
  #ifdef LPC_UART1
	if (UARTnum == 1) return uartPutch((LPC_UART_TypeDef *)LPC_UART1, ch);
  #endif	
  #ifdef LPC_UART2
	if (UARTnum == 2) return uartPutch((LPC_UART_TypeDef *)LPC_UART2, ch);
  #endif	
  #ifdef LPC_UART3
	if (UARTnum == 3) return uartPutch((LPC_UART_TypeDef *)LPC_UART3, ch);
  #endif	
	return -1;
}

int fputs(char *string, int UARTnum) {
	if (UARTnum == 0) return uartPuts((LPC_UART_TypeDef *)LPC_UART0, string);
  #ifdef LPC_UART1
	if (UARTnum == 1) return uartPuts((LPC_UART_TypeDef *)LPC_UART1, string);
  #endif	
  #ifdef LPC_UART2
	if (UARTnum == 2) return uartPuts((LPC_UART_TypeDef *)LPC_UART2, string);
  #endif	
  #ifdef LPC_UART3
	if (UARTnum == 3) return uartPuts((LPC_UART_TypeDef *)LPC_UART3, string);
  #endif	
	return -1;
}

